using System;
using System.IO.Ports;

namespace LeapCube
{
	public class Cube
	{
		public SerialPort serialPort;
		public static byte [,] frameBuffer;

		public Cube ()
		{
			serialPort = new SerialPort();
			ClearFrameBuffer ();
			#if DEBUG
			Console.WriteLine ("DataBits: " + serialPort.DataBits);
			Console.WriteLine ("Handshake: " + serialPort.Handshake);
			Console.WriteLine ("Portname: " + serialPort.PortName);
			#endif
		}

		public void Connect () 
		{
			serialPort.Open();
		}

		public void Disconnect () 
		{
			serialPort.Close();
		}

		public void ClearFrameBuffer () {
			frameBuffer = new byte[8,8];
		}

		public void WriteToFrameBuffer (byte x, byte y, byte z) {
			if (x < (byte)8 && y < (byte)8 && z < (byte)255) {
				if (frameBuffer [y, x] == 0)
					frameBuffer [y, x] = z;
				else 
					frameBuffer [y, x] |= z;
			}
		}

		public void CubePushFrameBuffer () {
			CubePush (frameBuffer);
		}

		public void CubePushCanvas (byte[,] canvas) {
			// frameBuffer = canvas;
			if (canvas.Length <= 64) {
				for (int i = 0; i < 8; i++) {
					for (int j = 0; j < 8; j++)
						frameBuffer[i,j] |= canvas[i,j];
				}
			}
				//CubePush (canvas);
			else
				Console.WriteLine ("Error: canvas array out of bounds.");
		}

		public void CubePush (byte[,] data){
			int x,y,i;

			i= 0;

			byte[] buffer = new byte[200];

			buffer[i++] = 0xff; // escape
			buffer[i++] = 0x00; // reset to 0,0

			for (x=0;x<8;x++)
			{
				for (y=0;y<8;y++)
				{
					buffer[i++] = data[x,y];
					if (data[x,y] == 0xff)
					{
						buffer[i++] = data[x,y];
					}
				}
			}

			serialPort.Write (buffer, 0, i);

		}

	}
}
