using System;
using Leap;

namespace LeapCube
{
	public class LeapListener : Listener
	{

		byte x, y, z = 0;
		Cube cube;
		byte[,] canvas = new byte[8,8];

		private Finger indexFinger = new Finger ();
		private Finger middleFinger = new Finger ();
		private Finger thumb = new Finger();

		private Object thisLock = new Object ();

		private void SafeWriteLine (String line)
		{
			lock (thisLock) {
				Console.WriteLine (line);
			}
		}

		public override void OnInit (Controller controller)
		{
			SafeWriteLine ("Initialized");
			cube = MainClass.cube;
		}

		public override void OnConnect (Controller controller)
		{
			SafeWriteLine ("Connected");
		}

		public override void OnDisconnect (Controller controller)
		{
			//Note: not dispatched when running in a debugger.
			SafeWriteLine ("Disconnected");
		}

		public override void OnExit (Controller controller)
		{
			SafeWriteLine ("Exited");
		}

		void DrawOnCanvas (byte x, byte y, byte z) {
			if (x < (byte)8 && y < (byte)8 && z < (byte)255) {
				canvas [y, x] |= z;
				SafeWriteLine ("canvas:\tx=" + x + "\ty=" + y + "\tz" + z);
				//SafeWriteLine ("canvas 0:0=" + canvas[0,0]);
			}
		}

		void EraseFromCanvas(byte x, byte y, byte z) {
			if (x < (byte)8 && y < (byte)8 && z < (byte)255) {
				z = (byte)~z;
				canvas [y, x] &= z;
				SafeWriteLine ("erasing");
			}
		}

		public override void OnFrame (Controller controller)
		{
			cube.ClearFrameBuffer ();


			// Get the most recent frame and report some basic information
			Frame frame = controller.Frame ();
			foreach (Hand hand in frame.Hands) {
				//cube.WriteToFrameBuffer (x, y, z);

				foreach (Finger finger in hand.Fingers) {
					if (finger.Type == Finger.FingerType.TYPE_INDEX)
						indexFinger = finger;
					if (finger.Type == Finger.FingerType.TYPE_MIDDLE)
						middleFinger = finger;
					if (finger.Type == Finger.FingerType.TYPE_THUMB)
						thumb = finger;

					x = (byte)Math.Floor((indexFinger.TipPosition.x + 90) * 0.044444444f);
					y = (byte)Math.Floor((indexFinger.TipPosition.y + 90 - MainClass.minDistance) * 0.044444444f);
					z = (byte)(1 << (byte)(Math.Floor(((indexFinger.TipPosition.z * -1 ) + 90) * 0.044444444f)));
					// SafeWriteLine ("temporary values:\tx=" + x + "\ty=" + y + "\tz=" + z);

					cube.WriteToFrameBuffer (x, y, z);
					//cube.CubePushFrameBuffer ();
				}
			}
			if (indexFinger.TipPosition.x - thumb.TipPosition.x < 20 && indexFinger.TipPosition.x - thumb.TipPosition.x > -20 &&
			    indexFinger.TipPosition.y - thumb.TipPosition.y < 20 && indexFinger.TipPosition.y - thumb.TipPosition.y > -20 &&
			    indexFinger.TipPosition.z - thumb.TipPosition.z < 20 && indexFinger.TipPosition.z - thumb.TipPosition.z > -20 &&
			    x < 8 && y < 8)
				DrawOnCanvas (x, y, z);

//			if (indexFinger.TipPosition.x - middleFinger.TipPosition.x < 2 &&
//			    indexFinger.TipPosition.y - middleFinger.TipPosition.y < 2 &&
//			    indexFinger.TipPosition.z - middleFinger.TipPosition.z < 2 )
//				EraseFromCanvas (x, y, z);

			if (indexFinger.TipPosition.x - middleFinger.TipPosition.x < 20 && indexFinger.TipPosition.x - middleFinger.TipPosition.x > -20 &&
			    indexFinger.TipPosition.y - middleFinger.TipPosition.y < 20 && indexFinger.TipPosition.y - middleFinger.TipPosition.y > -20 &&
			    indexFinger.TipPosition.z - middleFinger.TipPosition.z < 20 && indexFinger.TipPosition.z - middleFinger.TipPosition.z > -20 &&
			    x < 8 && y < 8)
				EraseFromCanvas (x, y, z);

			cube.CubePushCanvas (canvas);
			cube.CubePushFrameBuffer ();

		}
	}
}
