using System;
using Leap;

namespace LeapCube
{
	public class LeapListener : Listener
	{

		byte x, y, z = 0;
		Cube cube;

//		private Finger indexFinger = new Finger ();
//		private Finger middleFinger = new Finger ();

		private Object thisLock = new Object ();

		private void SafeWriteLine (String line)
		{
			lock (thisLock) {
				Console.WriteLine (line);
			}
		}

		public override void OnInit (Controller controller)
		{
			SafeWriteLine ("Initialized");
			cube = MainClass.cube;
		}

		public override void OnConnect (Controller controller)
		{
			SafeWriteLine ("Connected");
		}

		public override void OnDisconnect (Controller controller)
		{
			//Note: not dispatched when running in a debugger.
			SafeWriteLine ("Disconnected");
		}

		public override void OnExit (Controller controller)
		{
			SafeWriteLine ("Exited");
		}

		public override void OnFrame (Controller controller)
		{
			// Get the most recent frame and report some basic information
			Frame frame = controller.Frame ();
			Bone bone;

			cube.ClearFrameBuffer ();

			foreach (Hand hand in frame.Hands) {
				x = (byte)Math.Floor((hand.PalmPosition.x + 90) * 0.044444444f);
				y = (byte)Math.Floor((hand.PalmPosition.y + 90 - MainClass.minDistance) * 0.044444444f);
				z = (byte)(1 << (byte)(Math.Floor(((hand.PalmPosition.z * -1 ) + 90) * 0.044444444f)));
				cube.WriteToFrameBuffer (x, y, z);
				foreach (Finger finger in hand.Fingers) {
					x = (byte)Math.Floor((finger.TipPosition.x + 90) * 0.044444444f);
					y = (byte)Math.Floor((finger.TipPosition.y + 90 - MainClass.minDistance) * 0.044444444f);
					z = (byte)(1 << (byte)(Math.Floor(((finger.TipPosition.z * -1 ) + 90) * 0.044444444f)));
					// SafeWriteLine ("temporary values:\tx=" + x + "\ty=" + y + "\tz=" + z);
					cube.WriteToFrameBuffer (x, y, z);

					if (MainClass.showBones) {
					
						foreach (Bone.BoneType boneType in (Bone.BoneType[]) Enum.GetValues(typeof(Bone.BoneType))) {
							bone = finger.Bone (boneType);
							x = (byte)Math.Floor ((bone.Center.x + 90) * 0.044444444f);
							y = (byte)Math.Floor ((bone.Center.y + 90 - MainClass.minDistance) * 0.044444444f);
							z = (byte)(1 << (byte)(Math.Floor (((bone.Center.z * -1) + 90) * 0.044444444f)));
							// SafeWriteLine ("temporary values:\tx=" + x + "\ty=" + y + "\tz=" + z);
							cube.WriteToFrameBuffer (x, y, z);
						}
					}
				}
			}


			cube.CubePushFrameBuffer ();
		}
	}
}

