using System;
using Leap;
using System.Timers;

namespace LeapCube
{
	public class LeapListener : Listener
	{

		byte x, y, z = 0;
		Cube cube;
		Rain rain;
		private Timer timer = new Timer();

		//		private Finger indexFinger = new Finger ();
		//		private Finger middleFinger = new Finger ();

		private Object thisLock = new Object ();

		private void SafeWriteLine (String line)
		{
			lock (thisLock) {
				Console.WriteLine (line);
			}
		}

		private void SetTimer()
		{
			// Create a timer with a two second interval.
			timer = new Timer(50);
			// Hook up the Elapsed event for the timer. 
			timer.Elapsed += OnTimedEvent;
			timer.AutoReset = true;
			timer.Enabled = true;
		}

		private void OnTimedEvent(Object source, ElapsedEventArgs e)
		{
			rain.Raining ();
		}

		public override void OnInit (Controller controller)
		{
			cube = MainClass.cube;
			rain = new Rain();
			SafeWriteLine ("Initialized");
		}

		public override void OnConnect (Controller controller)
		{
			SafeWriteLine ("Connected");
			SetTimer ();
		}

		public override void OnDisconnect (Controller controller)
		{
			//Note: not dispatched when running in a debugger.
			SafeWriteLine ("Disconnected");
		}

		public override void OnExit (Controller controller)
		{
			cube.ClearFrameBuffer ();
			cube.CubePushFrameBuffer ();
			timer.Stop();
			timer.Dispose();
			SafeWriteLine ("Exited");
		}

		public override void OnFrame (Controller controller)
		{
			// Get the most recent frame and report some basic information
			Frame frame = controller.Frame ();
			Bone bone;
			rain.raincloud.Clear ();
//			cube.ClearFrameBuffer ();

			foreach (Hand hand in frame.Hands) {
				x = (byte)Math.Floor((hand.PalmPosition.x - 90) * -0.044444444f);
				y = (byte)Math.Floor((hand.PalmPosition.y + 90 - MainClass.minDistance) * 0.044444444f);
				z = (byte)(1 << (byte)(Math.Floor(((hand.PalmPosition.z * -1 ) - 90) * -0.044444444f)));
				cube.WriteToFrameBuffer (x, y, z);
				foreach (Finger finger in hand.Fingers) {
					x = (byte)Math.Floor((finger.TipPosition.x - 90) * -0.044444444f);
//					y = (byte)Math.Floor((finger.TipPosition.y + 90 - MainClass.minDistance) * 0.044444444f);
					// Draw y only on the 8th level.
					y = 7;
					z = (byte)(1 << (byte)(Math.Floor(((finger.TipPosition.z * -1 ) - 90) * -0.044444444f)));
					// SafeWriteLine ("temporary values:\tx=" + x + "\ty=" + y + "\tz=" + z);
					if (!Rain.pullStopper) {
						cube.WriteToFrameBuffer (x, y, z);
						rain.raincloud.Add (new byte[] { x, z });
					} else 
						Rain.stopperPulled = true;
				}
			}

			cube.CubePushFrameBuffer ();
		}
	}
}
