using System;
using System.Collections.Generic;
using System.Timers;

namespace LeapCube
{

	public class Rain
	{

		public int water = 0;
		public byte waterLevel = 0; // for debugging. default: 0
		public List<byte[]> raincloud; 
		public bool canRain = true;
		Mesh bubble;
		Mesh fish;
		Mesh vortex;
		static List<Mesh> bubbles = new List<Mesh>();
		static List<Mesh> fishes = new List<Mesh>();
		private int bubbleCount = 0;
		private Random rnd;
		private int iterations;
		private static Boolean spawnFish = false;

		private static Timer fishSpawnTimer = new Timer();
		private static Timer vortexTimer = new Timer();
		private static int fishSpawnDelay = 5000;
		public static Boolean pullStopper = false;
		public static Boolean stopperPulled = false;
		private float vortexHeight = 8;
		private float vortexSpeed = 2;

		public Rain ()
		{
			raincloud = new List<byte[]> ();
			rnd = new Random ();
			bubble = MeshImporter.Import ("sphere.obj");
			fish = MeshImporter.Import("fish.obj");
			// Pre-position the fish for the last animation.
			fish.pos_x = 4;
			fish.pos_y = -4;
			fish.pos_z = 4;
			fish.rot_x = 90;
			fish.rot_z =-90;

			vortex = MeshImporter.Import ("vortex.obj");
			iterations = 0;
		}

		public void Raining () {
			if (canRain) {
				Draw.clearplane_z (7); // Temporary fix for last row
				if (waterLevel <= 6) {

					waterLevel = (byte)(water / 32);
					Draw.shift (axis.AXIS_Z, -1, waterLevel, 6, true);
					if (waterLevel > 0) {
						Draw.setplane_z (waterLevel);
						for (int i = waterLevel -1; i >= 0; i--)
							Draw.clearplane_z (waterLevel - i - 1);
					}

					if (raincloud.Count > 2) {
						byte[] raindrop = raincloud [rnd.Next (raincloud.Count)];
						iterations++;
						if (iterations % 3 == 0) { // Don't let it rain to fast.
							MainClass.cube.WriteToFrameBuffer (raindrop [0], 6, raindrop [1]); 
							water = (water + 1) % 512;
							iterations = 0;
						}
					}

				} else {
					Draw.shift (axis.AXIS_Z, 1, 0, 7, false);
					canRain = false;
					SetTimer ();
				}
			} else {
				if (!stopperPulled)
					MakeBubbles (21);
				else
					ShowVortex ();
			}
		}

		private static void SetTimer()
		{
			fishSpawnTimer = new Timer(fishSpawnDelay);
			if (stopperPulled)
				vortexTimer = new Timer (50);

			// Hook up the Elapsed event for the timer. 
			fishSpawnTimer.Elapsed += OnTimedEvent;
			fishSpawnTimer.AutoReset = true;
			fishSpawnTimer.Enabled = true;
		}

		private static void OnTimedEvent(Object source, ElapsedEventArgs e)
		{
//			Console.WriteLine ("fish");
			spawnFish = true;
			if (!pullStopper)
				pullStopper = true;
		}

		private void MakeBubbles (int delay) {

			iterations++;
			if (iterations % delay == 0) { 
				bubbles.Add (new Mesh (bubble.vertices, 0.0000001f, rnd.Next (8), 0, rnd.Next (8), 0, 0, 0)); 
				iterations = 0;
			}

			// spawn a fish
			if (spawnFish) {
				int dir = rnd.Next(2);
				if (dir == 1)
					fishes.Add (new Mesh (fish.vertices, 0.0000015f, -4, rnd.Next(3,6), rnd.Next (8), 90, 180, 0)); 
				else 
					fishes.Add (new Mesh (fish.vertices, 0.0000015f, 12, rnd.Next(3,6), rnd.Next (8), 90, 0, 0)); 
				delay = 99; // Increase delay to incrase framerate.
				fishSpawnTimer.Interval = 2000;
				spawnFish = false;
			}

			// Move bubbles.
			if (iterations % 3 == 0) { // Slow it down.
				MainClass.cube.ClearFrameBuffer ();

				for (int i = 0; i < bubbles.Count; i++) {
//				Console.WriteLine (bubbles[i].pos_x + ":" + bubbles[i].pos_y + ":" + bubbles[i].pos_z);
					foreach (Vertex v in bubbles[i].vertices) {
						MainClass.cube.FBWrite ((byte)(bubbles [i].pos_x + v.x * bubbles [i].scale), (byte)(bubbles [i].pos_y + v.y * bubbles [i].scale), (byte)(bubbles [i].pos_z + v.z * bubbles [i].scale));
					}
					// Move bubbles upwards.
					bubbles [i] = new Mesh (bubbles [i].vertices, bubbles [i].scale + 0.0000003f, bubbles [i].pos_x, bubbles [i].pos_y + 1, bubbles [i].pos_z, bubbles [i].rot_x, bubbles [i].rot_y, bubbles [i].rot_z);
					if (bubbles [i].pos_y > 8)
						bubbles.Remove (bubbles [i]);
				}

				// Move fishes
				for (int i = 0; i < fishes.Count; i++) {
					// Draw fishes.
					foreach (Vertex v in fishes[i].vertices) {
						Vertex vOut = Graphx3D.Rotate (v, new Vertex(fishes[i].rot_x, fishes[i].rot_y, fishes[i].rot_z));
						MainClass.cube.FBWrite ((byte)(fishes[i].pos_x + vOut.x * fishes[i].scale),
						                        (byte)(fishes[i].pos_y + vOut.y * fishes[i].scale),
						                        (byte)(fishes[i].pos_z + vOut.z * fishes[i].scale)
						                        );
					}

					int dir = 1;
					if (fishes[i].rot_y == 0)
						dir = -1;

					if (fishes[i].pos_x <= 12 && fishes[i].pos_x >= -4)
						fishes[i] = new Mesh (fishes [i].vertices, fishes [i].scale, fishes [i].pos_x + dir, fishes [i].pos_y, fishes [i].pos_z, fishes [i].rot_x, fishes [i].rot_y, fishes [i].rot_z);
					else 
						fishes.RemoveAt(i);
//					Console.WriteLine ("fishes:" + fishes.Count);
				}
			}
		}

		private void ShowVortex () {
			if (vortexHeight > - 24) {
				MainClass.cube.ClearFrameBuffer ();
				foreach (Vertex v in vortex.vertices) {
					Vertex vOut = Graphx3D.Rotate (v, axis.AXIS_Y, vortex.rot_y);
					MainClass.cube.FBWrite ((byte)(4 + vOut.x * 0.000005f),
					                        (byte)(vortexHeight + vOut.y * 0.000005f),
					                        (byte)(4 + vOut.z * 0.000005f)
					);
				}
				vortexHeight -= 0.05f;
				vortexSpeed += 0.05f;
				vortex.rot_y += vortexSpeed;
			} else
				flyingFish ();
		}

		private void flyingFish () {

			MainClass.cube.ClearFrameBuffer ();

			foreach (Vertex v in fish.vertices) {
				Vertex vOut = Graphx3D.Rotate (v, new Vertex(fish.rot_x, fish.rot_y, fish.rot_z));
				MainClass.cube.FBWrite ((byte)(fish.pos_x + vOut.x * 0.0000015f),
				                        (byte)(fish.pos_y + vOut.y * 0.0000015f),
				                        (byte)(fish.pos_z + vOut.z * 0.0000015f)
				                        );
			}

			if (fish.pos_y < 16) {
				if (fish.pos_y < 4) 
					fish.pos_y += 0.25f;
				else if (fish.rot_z < 990) 
					fish.rot_z += 15;
				else
					fish.pos_y += 0.25f;
			} else 
				MainClass.Quit ();
		}

//		private void showFish() {
//			foreach (Vertex v in fish.vertices) {
//				Vertex vOut = Graphx3D.Rotate (v, new Vertex(90, fish.rot_y, fish.rot_z));
//				MainClass.cube.FBWrite ((byte)(4 + vOut.x * 0.0000015f),
//				                        (byte)(4 + vOut.y * 0.0000015f),
//				                        (byte)(4 + vOut.z * 0.0000015f)
//				                        );
//			}
//		}
			// rotation
			//newX = centerX + ( cos(α) * (point2X-centerX) - sin(α) * (point2Y -centerY))
			//newY = centerY + ( sin(α) * (point2X-centerX) + cos(α) * (point2Y -centerY))
	}
}

